@@ -27,12 +27,9 @@
 #include <glib-object.h>
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gtk/gtk.h>
 #include <clutter/clutter.h>
 #include <clutter-gst/clutter-gst.h>
-#include <gdk/gdkx.h>
 #include <gst/gst.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <X11/Xlib.h>
 
 #include "cheese-camera.h"
@@ -47,8 +44,6 @@ G_DEFINE_TYPE (CheeseCamera, cheese_camera, G_TYPE_OBJECT)
 
 typedef struct
 {
-  GtkWidget *video_window;
-
   GstElement *pipeline;
   GstBus *bus;
 
@@ -68,7 +63,7 @@ typedef struct
 
   ClutterTexture *video_texture;
 
-  GstElement *effect_filter, *csp_post_effect;
+  GstElement *effect_filter;
   GstElement *video_balance, *csp_post_balance;
   GstElement *camera_tee, *effects_tee;
   GstElement *effects_downscaler;
@@ -198,33 +193,62 @@ cheese_camera_bus_message_cb (GstBus *bus, GstMessage *message, CheeseCamera *ca
 {
   CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
 
-  if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_EOS)
+  switch (GST_MESSAGE_TYPE (message))
   {
-    if (priv->is_recording)
+    case GST_MESSAGE_EOS:
     {
-      GST_DEBUG ("Received EOS message");
-
-      g_source_remove (priv->eos_timeout_id);
-
-      /* emit signal by name here as the camera_signals array is empty in this thread */
-      /* TODO: really understand how threads and static works and why this is needed */
-      g_signal_emit_by_name (camera, "video-saved", NULL);
-
-      cheese_camera_change_sink (camera, priv->video_display_bin,
-                                 priv->photo_save_bin, priv->video_save_bin);
-      priv->is_recording = FALSE;
-    }
-  }
-  else if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_STATE_CHANGED)
-  {
-    if (strcmp (GST_MESSAGE_SRC_NAME (message), "pipeline") == 0)
-    {
-      GstState old, new;
-      gst_message_parse_state_changed (message, &old, &new, NULL);
-      if (new == GST_STATE_PLAYING)
+      if (priv->is_recording)
       {
-        g_signal_emit (camera, camera_signals[STATE_CHANGED], 0, new);
+        GST_DEBUG ("Received EOS message");
+
+        g_source_remove (priv->eos_timeout_id);
+
+        /* emit signal by name here as the camera_signals array is empty in this thread */
+        /* TODO: really understand how threads and static works and why this is needed */
+        g_signal_emit_by_name (camera, "video-saved", NULL);
+
+        cheese_camera_change_sink (camera, priv->video_display_bin,
+                                   priv->photo_save_bin, priv->video_save_bin);
+        priv->is_recording = FALSE;
       }
+
+    }
+    case GST_MESSAGE_WARNING:
+    {
+      GError *err;
+      gchar *debug;
+
+      gst_message_parse_warning (message, &err, &debug);
+      g_warning ("%s\n", err->message);
+      g_error_free (err);
+      g_free (debug);
+      break;
+    }
+    case GST_MESSAGE_ERROR:
+    {
+      GError *err;
+      gchar *debug;
+
+      gst_message_parse_error (message, &err, &debug);
+      g_warning ("%s\n", err->message);
+      g_error_free (err);
+      g_free (debug);
+      break;
+    }
+    case GST_MESSAGE_STATE_CHANGED:
+    {
+      if (strcmp (GST_MESSAGE_SRC_NAME (message), "pipeline") == 0)
+      {
+        GstState old, new;
+        gst_message_parse_state_changed (message, &old, &new, NULL);
+        if (new == GST_STATE_PLAYING)
+          g_signal_emit (camera, camera_signals[STATE_CHANGED], 0, new);
+      }
+      break;
+    }
+    default:
+    {
+      break;
     }
   }
 }
@@ -404,10 +428,6 @@ cheese_camera_create_video_display_bin (CheeseCamera *camera, GError **error)
   {
     cheese_camera_set_error_element_not_found (error, "identity");
   }
-  if ((priv->csp_post_effect = gst_element_factory_make ("ffmpegcolorspace", "csp_post_effect")) == NULL)
-  {
-    cheese_camera_set_error_element_not_found (error, "ffmpegcolorspace");
-  }
   if ((priv->video_balance = gst_element_factory_make ("videobalance", "video_balance")) == NULL)
   {
     cheese_camera_set_error_element_not_found (error, "videobalance");
@@ -456,14 +476,13 @@ cheese_camera_create_video_display_bin (CheeseCamera *camera, GError **error)
   gst_bin_add_many (GST_BIN (priv->video_display_bin), priv->camera_source_bin,
                     priv->camera_tee, priv->effects_downscaler, priv->effects_tee,
                     priv->effects_valve, priv->main_valve,
-                    priv->effect_filter, priv->csp_post_effect,
+                    priv->effect_filter,
                     priv->video_balance, priv->csp_post_balance,
                     save_tee, save_queue,
                     video_display_queue, video_scale, video_sink, NULL);
 
   ok  = gst_element_link_many (priv->camera_source_bin, priv->camera_tee, NULL);
   ok &= gst_element_link_many (priv->camera_tee, priv->main_valve, priv->effect_filter,
-                               priv->csp_post_effect,
                                priv->video_balance, priv->csp_post_balance,
                                save_tee, NULL);
   ok &= gst_element_link_many (priv->camera_tee, priv->effects_valve,
@@ -643,7 +662,7 @@ cheese_camera_create_video_save_bin (CheeseCamera *camera, GError **error)
   return TRUE;
 }
 
-int
+static int
 cheese_camera_get_num_camera_devices (CheeseCamera *camera)
 {
   CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
@@ -663,7 +682,7 @@ cheese_camera_get_selected_device (CheeseCamera *camera)
     return NULL;
 }
 
-gboolean
+void
 cheese_camera_switch_camera_device (CheeseCamera *camera)
 {
   CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
@@ -694,8 +713,6 @@ cheese_camera_switch_camera_device (CheeseCamera *camera)
    * {
    * Restart recording... ?
    * } */
-
-  return TRUE;
 }
 
 void
@@ -747,7 +764,7 @@ cheese_camera_change_effect_filter (CheeseCamera *camera, GstElement *new_filter
   g_object_set (G_OBJECT (priv->main_valve), "drop", TRUE, NULL);
 
   gst_element_unlink_many (priv->main_valve, priv->effect_filter,
-                           priv->csp_post_effect, NULL);
+                           priv->video_balance, NULL);
 
   g_object_ref (priv->effect_filter);
   gst_bin_remove (GST_BIN (priv->video_display_bin), priv->effect_filter);
@@ -756,7 +773,7 @@ cheese_camera_change_effect_filter (CheeseCamera *camera, GstElement *new_filter
 
   gst_bin_add (GST_BIN (priv->video_display_bin), new_filter);
   ok = gst_element_link_many (priv->main_valve, new_filter,
-                              priv->csp_post_effect, NULL);
+                              priv->video_balance, NULL);
   gst_element_set_state (new_filter, GST_STATE_PAUSED);
 
   g_return_if_fail (ok);
@@ -766,7 +783,7 @@ cheese_camera_change_effect_filter (CheeseCamera *camera, GstElement *new_filter
   priv->effect_filter = new_filter;
 }
 
-GstElement *
+static GstElement *
 cheese_camera_element_from_effect (CheeseCamera *camera, CheeseEffect *effect)
 {
   char       *effects_pipeline_desc;
@@ -850,6 +867,7 @@ cheese_camera_connect_effect_texture (CheeseCamera *camera, CheeseEffect *effect
 
 
   ok = gst_element_link_many (priv->effects_tee, control_valve, effect_filter, display_queue, display_element, NULL);
+  g_return_if_fail (ok);
 
   /* HACK: I don't understand GStreamer enough to know why this works. */
   gst_element_set_state (control_valve, GST_STATE_PLAYING);
@@ -861,7 +879,7 @@ cheese_camera_connect_effect_texture (CheeseCamera *camera, CheeseEffect *effect
 }
 
 void
-cheese_camera_start_video_recording (CheeseCamera *camera, char *filename)
+cheese_camera_start_video_recording (CheeseCamera *camera, const char *filename)
 {
   CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
 
@@ -913,7 +931,7 @@ cheese_camera_stop_video_recording (CheeseCamera *camera)
 }
 
 gboolean
-cheese_camera_take_photo (CheeseCamera *camera, char *filename)
+cheese_camera_take_photo (CheeseCamera *camera, const char *filename)
 {
   CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
 
@@ -1019,9 +1037,6 @@ cheese_camera_set_property (GObject *object, guint prop_id, const GValue *value,
   {
     case PROP_VIDEO_TEXTURE:
       priv->video_texture = g_value_get_pointer (value);
-
-      /*      g_signal_connect (priv->video_window, "expose-event", */
-      /*                G_CALLBACK (cheese_camera_expose_cb), self); */
       break;
     case PROP_DEVICE_NAME:
       g_free (priv->device_name);
@@ -1143,8 +1158,37 @@ cheese_camera_new (ClutterTexture *video_texture, char *camera_device_name,
   return camera;
 }
 
+
 void
-cheese_camera_setup (CheeseCamera *camera, char *id, GError **error)
+cheese_camera_set_device_by_dev_file (CheeseCamera *camera, const gchar *file)
+{
+  g_return_if_fail (CHEESE_IS_CAMERA (camera));
+  g_object_set (camera, "device_name", file, NULL);
+}
+
+static void
+cheese_camera_set_device_by_dev_udi (CheeseCamera *camera, const gchar *udi)
+{
+  CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
+  int                  i;
+
+  g_return_if_fail (CHEESE_IS_CAMERA (camera));
+
+
+  for (i = 0; i < priv->num_camera_devices; i++)
+  {
+    CheeseCameraDevice *device = g_ptr_array_index (priv->camera_devices, i);
+    if (strcmp (cheese_camera_device_get_id (device), udi) == 0)
+    {
+      g_object_set (camera,
+                    "device_name", cheese_camera_device_get_id (device),
+                    NULL);
+      break;
+    }
+  }
+}
+void
+cheese_camera_setup (CheeseCamera *camera, const char *id, GError **error)
 {
   CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
 
@@ -1206,35 +1250,6 @@ cheese_camera_get_camera_devices (CheeseCamera *camera)
   return g_ptr_array_ref (priv->camera_devices);
 }
 
-void
-cheese_camera_set_device_by_dev_file (CheeseCamera *camera, char *file)
-{
-  g_return_if_fail (CHEESE_IS_CAMERA (camera));
-  g_object_set (camera, "device_name", file, NULL);
-}
-
-void
-cheese_camera_set_device_by_dev_udi (CheeseCamera *camera, char *udi)
-{
-  CheeseCameraPrivate *priv;
-  int                  i;
-
-  g_return_if_fail (CHEESE_IS_CAMERA (camera));
-
-  priv = CHEESE_CAMERA_GET_PRIVATE (camera);
-
-  for (i = 0; i < priv->num_camera_devices; i++)
-  {
-    CheeseCameraDevice *device = g_ptr_array_index (priv->camera_devices, i);
-    if (strcmp (cheese_camera_device_get_id (device), udi) == 0)
-    {
-      g_object_set (camera,
-                    "device_name", cheese_camera_device_get_id (device),
-                    NULL);
-      break;
-    }
-  }
-}
 
 GList *
 cheese_camera_get_video_formats (CheeseCamera *camera)
@@ -1251,7 +1266,7 @@ cheese_camera_get_video_formats (CheeseCamera *camera)
     return NULL;
 }
 
-gboolean
+static gboolean
 cheese_camera_is_playing (CheeseCamera *camera)
 {
   CheeseCameraPrivate *priv;
@@ -1297,7 +1312,7 @@ cheese_camera_get_current_video_format (CheeseCamera *camera)
 
 gboolean
 cheese_camera_get_balance_property_range (CheeseCamera *camera,
-                                          gchar *property,
+                                          const gchar *property,
                                           gdouble *min, gdouble *max, gdouble *def)
 {
   CheeseCameraPrivate *priv;
@@ -1326,7 +1341,7 @@ cheese_camera_get_balance_property_range (CheeseCamera *camera,
 }
 
 void
-cheese_camera_set_balance_property (CheeseCamera *camera, gchar *property, gdouble value)
+cheese_camera_set_balance_property (CheeseCamera *camera, const gchar *property, gdouble value)
 {
   CheeseCameraPrivate *priv;
 
